@@ -448,7 +448,7 @@ int remove_record_from_server(uint32_t handle)
 }
 
 // FIXME: refactor for server-side
-static sdp_record_t *extract_pdu_server(bdaddr_t *device, uint8_t *p, int bufsize, uint32_t handleExpected, int *scanned)
+static sdp_record_t *extract_pdu_server(bdaddr_t *device, uint8_t *p, unsigned int bufsize, uint32_t handleExpected, int *scanned)
 {
 	int extractStatus = -1, localExtractedLength = 0;
 	uint8_t dtd;
@@ -606,6 +606,9 @@ success:
 	bt_put_unaligned(htonl(rec->handle), (uint32_t *) rsp->data);
 	rsp->data_size = sizeof(uint32_t);
 
+	if (rec)
+		sdp_record_free(rec);
+
 	return 0;
 
 invalid:
@@ -647,10 +650,11 @@ int service_update_req(sdp_req_t *req, sdp_buf_t *rsp)
 			SDPDBG("SvcRecOld : %p", orec);
 			SDPDBG("Failure to update, restore old value");
 
-			if (nrec)
-				sdp_record_free(nrec);
 			status = SDP_INVALID_SYNTAX;
 		}
+
+		if (nrec)
+			sdp_record_free(nrec);
 	} else
 		status = SDP_INVALID_RECORD_HANDLE;
 
